@@ -16,6 +16,7 @@
 
 ;;; Run the cross-compiler to produce cold fasl files.
 (setq sb!c::*track-full-called-fnames* :minimal) ; Change this as desired
+(setq sb!c::*static-vop-usage-counts* (make-hash-table))
 (let (fail
       variables
       functions
@@ -40,6 +41,19 @@
              ~:[~;functions (incomplete SB-COLD::*UNDEFINED-FUN-WHITELIST*?)~]"
             variables types functions)))
 
+(do-all-symbols (s)
+  (when (and (sb!int:info :function :inlinep s)
+             (eq (sb!int:info :function :where-from s) :assumed))
+      (warn "Did you forget to define ~S?" s)))
+
+;; enable this too see which vops were or weren't used
+#+nil
+(when (hash-table-p sb!c::*static-vop-usage-counts*)
+  (format t "Vops used:~%")
+  (dolist (cell (sort (sb!int:%hash-table-alist sb!c::*static-vop-usage-counts*)
+                      #'> :key #'cdr))
+    (format t "~6d ~s~%" (cdr cell) (car cell))))
+
 (when sb!c::*track-full-called-fnames*
   (let (possibly-suspicious likely-suspicious)
     (sb!int:call-with-each-globaldb-name
@@ -133,7 +147,8 @@ Sample output
 ;;; time to run it. The resulting core isn't used in the normal build,
 ;;; but can be handy for experimenting with the system. (See slam.sh
 ;;; for an example.)
-(when (position :sb-after-xc-core *shebang-features*)
+#+sb-after-xc-core
+(progn
   #+cmu (ext:save-lisp "output/after-xc.core" :load-init-file nil)
   #+sbcl (sb-ext:save-lisp-and-die "output/after-xc.core")
   #+openmcl (ccl::save-application "output/after-xc.core")
