@@ -21,38 +21,35 @@
 (define-assembly-routine
     (return-multiple (:return-style :none))
     (;; These are really arguments.
-     (:temp ecx unsigned-reg rcx-offset)
-     (:temp esi unsigned-reg rsi-offset)
+     (:temp rcx unsigned-reg rcx-offset)
+     (:temp rsi unsigned-reg rsi-offset)
 
      ;; These we need as temporaries.
-     (:temp eax unsigned-reg rax-offset)
-     (:temp ebx unsigned-reg rbx-offset)
-     (:temp edx unsigned-reg rdx-offset)
-     (:temp edi unsigned-reg rdi-offset)
+     (:temp rax unsigned-reg rax-offset)
+     (:temp rbx unsigned-reg rbx-offset)
+     (:temp rdx unsigned-reg rdx-offset)
+     (:temp rdi unsigned-reg rdi-offset)
      (:temp temp unsigned-reg r8-offset)
      (:temp loop-index unsigned-reg r9-offset))
 
   ;; Pick off the cases where everything fits in register args.
   (inst jrcxz ZERO-VALUES)
-  (inst cmp ecx (fixnumize 1))
+  (inst cmp rcx (fixnumize 1))
   (inst jmp :e ONE-VALUE)
-  (inst cmp ecx (fixnumize 2))
+  (inst cmp rcx (fixnumize 2))
   (inst jmp :e TWO-VALUES)
-  (inst cmp ecx (fixnumize 3))
+  (inst cmp rcx (fixnumize 3))
   (inst jmp :e THREE-VALUES)
 
-  ;; As per the calling convention EBX is expected to point at the SP
+  ;; As per the calling convention RBX is expected to point at the SP
   ;; before the stack frame.
-  (inst lea ebx (make-ea :qword :base rbp-tn
-                         :disp (* sp->fp-offset n-word-bytes)))
+  (inst lea rbx (ea (* sp->fp-offset n-word-bytes) rbp-tn))
 
   ;; Save the count, the return address and restore the frame pointer,
   ;; because the loop is going to destroy them.
-  (inst mov edx ecx)
-  (inst mov eax (make-ea :qword :base rbp-tn
-                         :disp (frame-byte-offset return-pc-save-offset)))
-  (inst mov rbp-tn (make-ea :qword :base rbp-tn
-                            :disp (frame-byte-offset ocfp-save-offset)))
+  (inst mov rdx rcx)
+  (inst mov rax (ea (frame-byte-offset return-pc-save-offset) rbp-tn))
+  (inst mov rbp-tn (ea (frame-byte-offset ocfp-save-offset) rbp-tn))
   ;; Blit the values down the stack. Note: there might be overlap, so
   ;; we have to be careful not to clobber values before we've read
   ;; them. Because the stack builds down, we are copying to a larger
@@ -61,37 +58,31 @@
   (zeroize loop-index)
   LOOP
   (inst sub loop-index n-word-bytes)
-  (inst mov temp
-        (make-ea :qword :base esi
-                        :index loop-index))
-  (inst mov
-        (make-ea :qword :base ebx
-                        :index loop-index)
-        temp)
+  (inst mov temp (ea rsi loop-index))
+  (inst mov (ea rbx loop-index) temp)
 
-  (inst sub edx (fixnumize 1))
+  (inst sub rdx (fixnumize 1))
   (inst jmp :nz LOOP)
 
   ;; Set the stack top to the last result.
-  (inst lea rsp-tn (make-ea :qword :base ebx :index loop-index))
+  (inst lea rsp-tn (ea rbx loop-index))
 
   ;; Load the register args.
-  (loadw edx ebx -1)
-  (loadw edi ebx -2)
-  (loadw esi ebx -3)
+  (loadw rdx rbx -1)
+  (loadw rdi rbx -2)
+  (loadw rsi rbx -3)
 
   ;; And back we go.
   (inst stc)
-  (inst push eax)
+  (inst push rax)
   (inst ret)
 
   ;; Handle the register arg cases.
   ZERO-VALUES
-  (inst lea ebx (make-ea :qword :base rbp-tn
-                         :disp (* sp->fp-offset n-word-bytes)))
-  (inst mov edx nil-value)
-  (inst mov edi edx)
-  (inst mov esi edx)
+  (inst lea rbx (ea (* sp->fp-offset n-word-bytes) rbp-tn))
+  (inst mov rdx nil-value)
+  (inst mov rdi rdx)
+  (inst mov rsi rdx)
   (inst mov rsp-tn rbp-tn)
   (inst stc)
   (inst pop rbp-tn)
@@ -100,29 +91,27 @@
   ;; Note: we can get this, because the return-multiple vop doesn't
   ;; check for this case when size > speed.
   ONE-VALUE
-  (loadw edx esi -1)
+  (loadw rdx rsi -1)
   (inst mov rsp-tn rbp-tn)
   (inst clc)
   (inst pop rbp-tn)
   (inst ret)
 
   TWO-VALUES
-  (inst lea ebx (make-ea :qword :base rbp-tn
-                         :disp (* sp->fp-offset n-word-bytes)))
-  (loadw edx esi -1)
-  (loadw edi esi -2)
-  (inst mov esi nil-value)
+  (inst lea rbx (ea (* sp->fp-offset n-word-bytes) rbp-tn))
+  (loadw rdx rsi -1)
+  (loadw rdi rsi -2)
+  (inst mov rsi nil-value)
   (inst mov rsp-tn rbp-tn)
   (inst stc)
   (inst pop rbp-tn)
   (inst ret)
 
   THREE-VALUES
-  (inst lea ebx (make-ea :qword :base rbp-tn
-                         :disp (* sp->fp-offset n-word-bytes)))
-  (loadw edx esi -1)
-  (loadw edi esi -2)
-  (loadw esi esi -3)
+  (inst lea rbx (ea (* sp->fp-offset n-word-bytes) rbp-tn))
+  (loadw rdx rsi -1)
+  (loadw rdi rsi -2)
+  (loadw rsi rsi -3)
   (inst mov rsp-tn rbp-tn)
   (inst stc)
   (inst pop rbp-tn)
@@ -134,80 +123,66 @@
 ;;; of our stack frame (where args are produced) to the start of our
 ;;; stack frame (where args are expected).
 ;;;
-;;; We take the function to call in EAX and a pointer to the arguments in
-;;; ESI. EBP says the same over the jump, and the old frame pointer is
+;;; We take the function to call in RAX and a pointer to the arguments in
+;;; RSI. EBP stays the same over the jump, and the old frame pointer is
 ;;; still saved in the first stack slot. The return-pc is saved in
 ;;; the second stack slot, so we have to push it to make it look like
-;;; we actually called. We also have to compute ECX from the difference
-;;; between ESI and the stack top.
+;;; we actually called. We also have to compute RCX from the difference
+;;; between RSI and the stack top.
 #-sb-assembling ; avoid "Redefinition" warning (this file is processed twice)
-(defun !prepare-for-tail-call-variable (eax ebx ecx edx edi esi
+(defun !prepare-for-tail-call-variable (fun temp nargs rdx rdi rsi
+                                        r8 r9 r10
                                         &optional jump-to-the-end)
   (assemble ()
     ;; Calculate NARGS (as a fixnum)
-    (move ecx esi)
-    (inst sub ecx rsp-tn)
+    (move nargs rsi)
+    (inst sub nargs rsp-tn)
     #!-#.(cl:if (cl:= sb!vm:word-shift sb!vm:n-fixnum-tag-bits) '(and) '(or))
-    (inst shr ecx (- word-shift n-fixnum-tag-bits))
+    (inst shr nargs (- word-shift n-fixnum-tag-bits))
 
     ;; Check for all the args fitting the registers.
-    (inst cmp ecx (fixnumize register-arg-count))
+    (inst cmp nargs (fixnumize register-arg-count))
     (inst jmp :le REGISTER-ARGS)
 
-    ;; Save the OLD-FP and RETURN-PC because the blit is going to trash
-    ;; those stack locations. Save the ECX, because the loop is going to
-    ;; trash it.
-    (pushw rbp-tn (frame-word-offset ocfp-save-offset))
-    (loadw ebx rbp-tn (frame-word-offset return-pc-save-offset))
-    (inst push ecx)
+    (inst mov r8 rsi)
+
+    ;; Register args
+    (loadw rdx rsi -1)
+    (loadw rdi rsi -2)
+    (loadw rsi rsi -3)
 
     ;; Do the blit. Because we are coping from smaller addresses to
     ;; larger addresses, we have to start at the largest pair and work
     ;; our way down.
-    (inst shr ecx n-fixnum-tag-bits)
-    (inst std)                          ; count down
-    (inst lea edi (make-ea :qword :base rbp-tn :disp (frame-byte-offset 0)))
-    (inst sub esi n-word-bytes)
-    (inst rep)
-    (inst movs :qword)
-    (inst cld)
+    (zeroize temp)
+    (inst lea r9 (ea (- (fixnumize register-arg-count)) nargs))
+    LOOP
+    (inst sub temp n-word-bytes)
+    (inst mov r10 (ea (- (* register-arg-count n-word-bytes)) r8 temp))
+    (inst mov (ea (frame-byte-offset (1- register-arg-count)) rbp-tn temp) r10)
 
-    ;; Load the register arguments carefully.
-    (loadw edx rbp-tn (frame-word-offset ocfp-save-offset))
-
-    ;; Restore OLD-FP and ECX.
-    (inst pop ecx)
-    ;; Overwrites a1
-    (popw rbp-tn (frame-word-offset ocfp-save-offset))
+    (inst sub r9 (fixnumize 1))
+    (inst jmp :nz LOOP)
 
     ;; Blow off the stack above the arguments.
-    (inst lea rsp-tn (make-ea :qword :base edi :disp n-word-bytes))
+    (inst lea rsp-tn (ea (frame-byte-offset (1- register-arg-count)) rbp-tn temp))
 
-    ;; remaining register args
-    (inst mov edi edx)
-    (loadw edx rbp-tn (frame-word-offset 0))
-    (loadw esi rbp-tn (frame-word-offset 2))
-
-    ;; Push the (saved) return-pc so it looks like we just called.
-    (inst push ebx)
+    ;; Push the return-pc so it looks like we just called.
+    (pushw rbp-tn (frame-word-offset return-pc-save-offset))
 
     ;; And jump into the function.
     (if jump-to-the-end
         (inst jmp end)
-        (inst jmp
-              (make-ea :byte :base eax
-                             :disp (- (* closure-fun-slot n-word-bytes)
-                                      fun-pointer-lowtag))))
+        (inst jmp (ea (- (* closure-fun-slot n-word-bytes) fun-pointer-lowtag) fun)))
 
     ;; All the arguments fit in registers, so load them.
     REGISTER-ARGS
-    (loadw edx esi -1)
-    (loadw edi esi -2)
-    (loadw esi esi -3)
+    (loadw rdx rsi -1)
+    (loadw rdi rsi -2)
+    (loadw rsi rsi -3)
 
     ;; Clear most of the stack.
-    (inst lea rsp-tn
-          (make-ea :qword :base rbp-tn :disp (* (- sp->fp-offset 3) n-word-bytes)))
+    (inst lea rsp-tn (ea (* (- sp->fp-offset 3) n-word-bytes) rbp-tn))
 
     ;; Push the return-pc so it looks like we just called.
     (pushw rbp-tn (frame-word-offset return-pc-save-offset))
@@ -217,38 +192,38 @@
 (define-assembly-routine
     (tail-call-variable
      (:return-style :none))
-    ((:temp eax unsigned-reg rax-offset)
-     (:temp ebx unsigned-reg rbx-offset)
-     (:temp ecx unsigned-reg rcx-offset)
-     (:temp edx unsigned-reg rdx-offset)
-     (:temp edi unsigned-reg rdi-offset)
-     (:temp esi unsigned-reg rsi-offset))
-  (!prepare-for-tail-call-variable eax ebx ecx edx edi esi)
+    ((:temp fun unsigned-reg rax-offset)
+     (:temp temp unsigned-reg rbx-offset)
+     (:temp nargs unsigned-reg rcx-offset)
+     (:temp rdx unsigned-reg rdx-offset)
+     (:temp rdi unsigned-reg rdi-offset)
+     (:temp rsi unsigned-reg rsi-offset)
+     (:temp r8 unsigned-reg r8-offset)
+     (:temp r9 unsigned-reg r9-offset)
+     (:temp r10 unsigned-reg r10-offset))
+  (!prepare-for-tail-call-variable fun temp nargs rdx rdi rsi r8 r9 r10)
 
-  (inst jmp (make-ea :byte :base eax
-                     :disp (- (* closure-fun-slot n-word-bytes)
-                              fun-pointer-lowtag))))
+  (inst jmp (ea (- (* closure-fun-slot n-word-bytes) fun-pointer-lowtag) fun)))
 
 #+sb-assembling
 (define-assembly-routine
     (tail-call-callable-variable
      (:return-style :none))
+    ((:temp fun unsigned-reg rax-offset)
+     (:temp temp unsigned-reg rbx-offset)
+     (:temp nargs unsigned-reg rcx-offset)
+     (:temp rdx unsigned-reg rdx-offset)
+     (:temp rdi unsigned-reg rdi-offset)
+     (:temp rsi unsigned-reg rsi-offset)
+     (:temp r8 unsigned-reg r8-offset)
+     (:temp r9 unsigned-reg r9-offset)
+     (:temp r10 unsigned-reg r10-offset))
+  (!prepare-for-tail-call-variable fun temp nargs rdx rdi rsi r8 r9 r10 t)
 
-    ((:temp eax unsigned-reg rax-offset)
-     (:temp ebx unsigned-reg rbx-offset)
-     (:temp ecx unsigned-reg rcx-offset)
-     (:temp edx unsigned-reg rdx-offset)
-     (:temp edi unsigned-reg rdi-offset)
-     (:temp esi unsigned-reg rsi-offset)
-     (:temp fun (any-reg descriptor-reg) rax-offset))
-  (!prepare-for-tail-call-variable eax ebx ecx edx edi esi t)
-
-  (%lea-for-lowtag-test ebx-tn fun fun-pointer-lowtag)
+  (%lea-for-lowtag-test rbx-tn fun fun-pointer-lowtag)
   (inst test bl-tn lowtag-mask)
   (inst jmp :nz (make-fixup 'tail-call-symbol :assembly-routine))
-  (inst jmp (make-ea :byte :base eax
-                           :disp (- (* closure-fun-slot n-word-bytes)
-                                    fun-pointer-lowtag))))
+  (inst jmp (ea (- (* closure-fun-slot n-word-bytes) fun-pointer-lowtag) fun)))
 
 #+sb-assembling
 (define-assembly-routine (call-symbol
@@ -258,39 +233,31 @@
      (:temp length (any-reg descriptor-reg) rax-offset)
      (:temp vector (any-reg descriptor-reg) rbx-offset))
   ;; Jump over CALL QWORD PTR [RAX-3] in the caller
-  (inst add (make-ea :qword :base rsp-tn) 3)
+  (inst add :qword (ea rsp-tn) 3)
   (emit-alignment n-lowtag-bits :long-nop)
 
   TAIL-CALL-SYMBOL
   (%lea-for-lowtag-test vector fun other-pointer-lowtag)
-  (inst test (reg-in-size vector :byte) lowtag-mask)
+  (inst test :byte vector lowtag-mask)
   (inst jmp :nz not-callable)
-  (inst cmp (make-ea :byte :base fun :disp (- other-pointer-lowtag))
-        symbol-widetag)
+  (inst cmp :byte (ea (- other-pointer-lowtag) fun) symbol-widetag)
   (inst jmp :ne not-callable)
   (load-symbol-info-vector vector fun r11-tn)
   ;; info-vector-fdefn
   (inst cmp vector nil-value)
   (inst jmp :e undefined)
 
-  (inst mov r10d-tn (make-ea :dword :base vector
-                                    :disp (- (* 2 n-word-bytes) other-pointer-lowtag)))
+  (inst mov r10d-tn (ea (- (* 2 n-word-bytes) other-pointer-lowtag) vector))
   (inst and r10d-tn (fixnumize (1- (ash 1 (* info-number-bits 2)))))
   (inst cmp r10d-tn (fixnumize (1+ (ash +fdefn-info-num+ info-number-bits))))
   (inst jmp :b undefined)
 
   (loadw length vector 1 other-pointer-lowtag)
-  (inst mov fun (make-ea :qword :base vector
-                                :index length
-                                :scale 4
-                                :disp
-                                (- 8 other-pointer-lowtag)))
+  (inst mov fun (ea (- 8 other-pointer-lowtag) vector length 4))
 
   (let ((fdefn-raw-addr
-          (make-ea :qword :base fun
-                          :disp (- (* fdefn-raw-addr-slot
-                                      n-word-bytes)
-                                   other-pointer-lowtag))))
+          (ea (- (* fdefn-raw-addr-slot n-word-bytes) other-pointer-lowtag)
+              fun)))
     #!+immobile-code
     (progn
       (inst lea vector fdefn-raw-addr)
@@ -303,7 +270,7 @@
   (inst cmp fun nil-value) ;; NIL doesn't have SYMBOL-WIDETAG
   (inst jmp :e undefined)
 
-  (inst pop (make-ea :qword :base rbp-tn :disp n-word-bytes))
+  (inst pop (ea n-word-bytes rbp-tn))
   (emit-error-break nil error-trap (error-number-or-lose 'sb!kernel::object-not-callable-error)
                     (list fun)))
 
@@ -343,7 +310,7 @@
 
   EXIT
 
-  ;; Here EAX points to catch block containing symbol pointed to by EDX.
+  ;; Here RAX points to catch block containing symbol pointed to by RDX.
   ;; An extra RET gets stuffed after the JMP, but oh well. You can't just change
   ;; the :return-style to :none because that also affects the call sequence.
   (inst jmp (make-fixup 'unwind :assembly-routine)))
@@ -357,7 +324,12 @@
                          ((:arg block (any-reg descriptor-reg) rax-offset)
                           (:arg start (any-reg descriptor-reg) rbx-offset)
                           (:arg count (any-reg descriptor-reg) rcx-offset)
-                          (:temp uwp unsigned-reg rsi-offset))
+                          (:temp uwp unsigned-reg rsi-offset)
+                          ;; for unbind-to-here
+                          (:temp where unsigned-reg r8-offset)
+                          (:temp symbol unsigned-reg r9-offset)
+                          (:temp value unsigned-reg r10-offset)
+                          (:temp zero complex-double-reg float0-offset))
   (declare (ignore start count))
 
   (let ((error (generate-error-code nil 'invalid-unwind-error)))
@@ -385,12 +357,16 @@
 
   (loadw rbp-tn block unwind-block-cfp-slot)
 
-  ;; Uwp-entry expects some things in known locations so that they can
-  ;; be saved on the stack: the block in edx-tn, start in ebx-tn, and
-  ;; count in ecx-tn.
+  (loadw uwp block unwind-block-current-catch-slot)
+  (store-tl-symbol-value uwp *current-catch-block*)
 
-  (inst jmp (make-ea :byte :base block
-                     :disp (* unwind-block-entry-pc-slot n-word-bytes))))
+  (loadw where block unwind-block-bsp-slot)
+  (unbind-to-here where symbol value uwp zero)
+  ;; Uwp-entry expects some things in known locations so that they can
+  ;; be saved on the stack: the block in rdx-tn, start in rbx-tn, and
+  ;; count in rcx-tn.
+
+  (inst jmp (ea (* unwind-block-entry-pc-slot n-word-bytes) block)))
 
 ;;; Perform a store to code, updating the GC page (card) protection bits.
 ;;; This is not a "good" implementation of soft card marking.
@@ -420,26 +396,24 @@
     ;; because there's no way to get back into C code anyhow.
     #!+sb-dynamic-core
     (progn
-      (inst mov thread-base-tn
-            (make-ea :qword :disp (make-fixup "all_threads" :foreign-dataref)))
-      (inst mov thread-base-tn (make-ea :qword :base thread-base-tn)))
+      (inst mov thread-base-tn (ea (make-fixup "all_threads" :foreign-dataref)))
+      (inst mov thread-base-tn (ea thread-base-tn)))
     #!-sb-dynamic-core
-    (inst mov thread-base-tn
-          (make-ea :qword :disp (make-fixup "all_threads" :foreign))))
+    (inst mov thread-base-tn (ea (make-fixup "all_threads" :foreign))))
 
-  (inst mov temp-reg-tn (make-ea :qword :base rsp-tn :disp 24))
+  (inst mov temp-reg-tn (ea 24 rsp-tn))
   (inst sub temp-reg-tn (thread-slot-ea thread-varyobj-space-addr-slot))
   (inst shr temp-reg-tn (1- (integer-length immobile-card-bytes)))
-  (pseudo-atomic
+  (pseudo-atomic ()
     (assemble ()
       (inst cmp temp-reg-tn (thread-slot-ea thread-varyobj-card-count-slot))
       (inst jmp :ae try-dynamic-space)
       (inst mov rdi-tn (thread-slot-ea thread-varyobj-card-marks-slot))
-      (inst bts (make-ea :qword :base rdi-tn) temp-reg-tn :lock)
+      (inst bts (ea rdi-tn) temp-reg-tn :lock)
       (inst jmp store)
 
       TRY-DYNAMIC-SPACE
-      (inst mov temp-reg-tn (make-ea :qword :base rsp-tn :disp 24)) ; reload
+      (inst mov temp-reg-tn (ea 24 rsp-tn)) ; reload
       (inst sub temp-reg-tn (thread-slot-ea thread-dynspace-addr-slot))
       (inst shr temp-reg-tn (1- (integer-length gencgc-card-bytes)))
       (inst cmp temp-reg-tn (thread-slot-ea thread-dynspace-card-count-slot))
@@ -453,28 +427,23 @@
              (inst shl temp-reg-tn 3) ; multiply by 8
              (inst add temp-reg-tn (thread-slot-ea thread-dynspace-pte-base-slot))
              ;; clear WP - bit index 5 of flags byte
-             (inst and (make-ea :byte :base temp-reg-tn :disp 6) (lognot (ash 1 5))
-                   :lock))
+             (inst and :byte (ea 6 temp-reg-tn) (lognot (ash 1 5)) :lock))
             (t
-             (inst lea temp-reg-tn ; multiply by 3
-                   (make-ea :qword :base temp-reg-tn :index temp-reg-tn :scale 2))
+             (inst lea temp-reg-tn (ea temp-reg-tn temp-reg-tn 2)) ; multiply by 3
              (inst shl temp-reg-tn 2) ; then by 4, = 12
              (inst add temp-reg-tn (thread-slot-ea thread-dynspace-pte-base-slot))
              ;; clear WP
-             (inst and (make-ea :byte :base temp-reg-tn :disp 8) (lognot (ash 1 5))
-                   :lock)))
+             (inst and :byte (ea 8 temp-reg-tn) (lognot (ash 1 5)) :lock)))
 
       STORE
-      (inst mov rdi-tn (make-ea :qword :base rsp-tn :disp 24))      ; object
-      (inst mov temp-reg-tn (make-ea :qword :base rsp-tn :disp 32)) ; word index
-      (inst mov rax-tn (make-ea :qword :base rsp-tn :disp 40))      ; newval
+      (inst mov rdi-tn (ea 24 rsp-tn))      ; object
+      (inst mov temp-reg-tn (ea 32 rsp-tn)) ; word index
+      (inst mov rax-tn (ea 40 rsp-tn))      ; newval
       ;; set 'written' flag in the code header
-      (inst or (make-ea :byte :base rdi-tn :disp (- 3 other-pointer-lowtag))
-            #x40 :lock)
+      (inst or :byte (ea (- 3 other-pointer-lowtag) rdi-tn) #x40 :lock)
       ;; store newval into object
-      (inst mov (make-ea :qword :base rdi-tn
-                         :index temp-reg-tn :scale (ash 1 word-shift)
-                         :disp (- other-pointer-lowtag)) rax-tn)))
+      (inst mov (ea (- other-pointer-lowtag) rdi-tn temp-reg-tn n-word-bytes)
+            rax-tn)))
   (inst pop rdi-tn) ; restore
   (inst pop rax-tn)
   (inst ret 24)) ; remove 3 stack args
