@@ -500,9 +500,10 @@ between the ~A definition and the ~A definition"
 ;;; don't have a corresponding class.
 (def!struct (structure-classoid (:include classoid)
                                 (:copier nil)
-                                (:constructor %make-structure-classoid)))
+                                (:constructor %make-structure-classoid
+                                    (hash-value name))))
 (defun make-structure-classoid (&key name)
-  (mark-ctype-interned (%make-structure-classoid :name name)))
+  (%make-structure-classoid (interned-type-hash name) name))
 
 ;;;; classoid namespace
 
@@ -1118,15 +1119,15 @@ between the ~A definition and the ~A definition"
                     (t
                      (setf (classoid-cell-classoid
                             (find-classoid-cell name :create t))
-                           (mark-ctype-interned
-                            (make-built-in-classoid
+                           (!make-built-in-classoid
+                             :hash-value (interned-type-hash name)
                              :name name
                              :translation (if trans-p :initializing nil)
                              :direct-superclasses
                              (if (eq name t)
                                  nil
                                  (mapcar #'find-classoid
-                                         direct-superclasses)))))))))
+                                         direct-superclasses))))))))
         (setf (info :type :kind name) :primitive)
         (unless trans-p
           (setf (info :type :builtin name) classoid))
@@ -1214,21 +1215,21 @@ between the ~A definition and the ~A definition"
   #-sb-xc-host (/show0 "about to set *BUILT-IN-CLASS-CODES*")
   (setq **built-in-class-codes**
         (let* ((initial-element (classoid-layout (find-classoid 'random-class)))
-               (res (make-array 256 :initial-element initial-element)))
-          (dolist (x *!built-in-classes* res)
+               (table (make-array 256 :initial-element initial-element)))
+          (dolist (x *!built-in-classes*)
             (destructuring-bind (name &key codes &allow-other-keys)
                                 x
               (let ((layout (classoid-layout (find-classoid name))))
                 (dolist (code codes)
-                  (setf (svref res code) layout)))))))
-  #!+immobile-space
-  (let ((table **built-in-class-codes**))
-    (loop with layout = (aref table sb!vm:list-pointer-lowtag)
-          for i from sb!vm:list-pointer-lowtag by 16 below 256
-          do (setf (aref table i) layout))
-    (loop with layout = (aref table sb!vm:even-fixnum-lowtag)
-          for i from sb!vm:even-fixnum-lowtag by (ash 1 sb!vm:n-fixnum-tag-bits) below 256
-          do (setf (aref table i) layout)))
-  #-sb-xc-host (/show0 "done setting *BUILT-IN-CLASS-CODES*"))
+                  (setf (svref table code) layout)))))
+          (loop with layout = (aref table sb!vm:list-pointer-lowtag)
+                for i from sb!vm:list-pointer-lowtag by (* 2 sb!vm:n-word-bytes)
+                below 256
+                do (setf (aref table i) layout))
+          (loop with layout = (aref table sb!vm:even-fixnum-lowtag)
+                for i from sb!vm:even-fixnum-lowtag by (ash 1 sb!vm:n-fixnum-tag-bits)
+                below 256
+                do (setf (aref table i) layout))
+          table)))
 
 (!defun-from-collected-cold-init-forms !classes-cold-init)
