@@ -11,58 +11,67 @@
 
 (in-package "SB!VM")
 
-(defun %test-fixnum (value target not-p &key temp)
+(defun %test-fixnum (value temp target not-p)
   (declare (ignore temp))
   (assemble ()
     (inst tst value fixnum-tag-mask)
     (inst b (if not-p :ne :eq) target)))
 
-(defun %test-fixnum-immediate-and-headers (value target not-p immediate
-                                           headers &key temp)
+(defun %test-fixnum-immediate-and-headers (value temp target not-p immediate
+                                           headers &key value-tn-ref)
   (let ((drop-through (gen-label)))
     (inst tst value fixnum-tag-mask)
     (inst b :eq (if not-p drop-through target))
-    (%test-immediate-and-headers value target not-p immediate headers
-                                 :drop-through drop-through :temp temp)))
+    (%test-immediate-and-headers value temp target not-p immediate headers
+                                 :drop-through drop-through
+                                 :value-tn-ref value-tn-ref)))
 
-(defun %test-immediate-and-headers (value target not-p immediate headers
-                                    &key (drop-through (gen-label)) temp)
+(defun %test-immediate-and-headers (value temp target not-p immediate headers
+                                    &key (drop-through (gen-label))
+                                         value-tn-ref)
 
   (inst mov temp immediate)
   (inst cmp temp (extend value :uxtb))
   (inst b :eq (if not-p drop-through target))
-  (%test-headers value target not-p nil headers :drop-through drop-through
-                                                :temp temp))
+  (%test-headers value temp target not-p nil headers
+                 :drop-through drop-through
+                 :value-tn-ref value-tn-ref))
 
-(defun %test-fixnum-and-headers (value target not-p headers &key temp)
+(defun %test-fixnum-and-headers (value temp target not-p headers
+                                 &key value-tn-ref)
   (let ((drop-through (gen-label)))
     (assemble ()
       (inst ands temp value fixnum-tag-mask)
       (inst b :eq (if not-p drop-through target)))
-    (%test-headers value target not-p nil headers
-                   :drop-through drop-through :temp temp)))
+    (%test-headers value temp target not-p nil headers
+                   :drop-through drop-through
+                   :value-tn-ref value-tn-ref)))
 
-(defun %test-immediate (value target not-p immediate &key temp)
+(defun %test-immediate (value temp target not-p immediate)
   (assemble ()
     (inst and temp value widetag-mask)
     (inst cmp temp immediate)
     (inst b (if not-p :ne :eq) target)))
 
-(defun %test-lowtag (value target not-p lowtag &key temp)
+(defun %test-lowtag (value temp target not-p lowtag)
   (assemble ()
     (inst and temp value lowtag-mask)
     (inst cmp temp lowtag)
     (inst b (if not-p :ne :eq) target)))
 
-(defun %test-headers (value target not-p function-p headers
-                      &key temp (drop-through (gen-label)))
-    (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag)))
+(defun %test-headers (value temp target not-p function-p headers
+                      &key (drop-through (gen-label))
+                           value-tn-ref)
+  (let ((lowtag (if function-p fun-pointer-lowtag other-pointer-lowtag)))
     (multiple-value-bind (when-true when-false)
         (if not-p
             (values drop-through target)
             (values target drop-through))
       (assemble ()
-        (%test-lowtag value when-false t lowtag :temp temp)
+        (unless (and value-tn-ref
+                     (eq lowtag other-pointer-lowtag)
+                     (other-pointer-tn-ref-p value-tn-ref))
+          (%test-lowtag value temp when-false t lowtag))
         (load-type temp value (- lowtag))
         (do ((remaining headers (cdr remaining)))
             ((null remaining))
@@ -134,7 +143,7 @@
       (assemble ()
         (inst ands temp value fixnum-tag-mask)
         (inst b :eq yep)
-        (test-type value nope t (other-pointer-lowtag) :temp temp)
+        (test-type value temp nope t (other-pointer-lowtag))
         (loadw temp value 0 other-pointer-lowtag)
         (inst cmp temp (+ (ash 1 n-widetag-bits) bignum-widetag))
         (inst b (if not-p :ne :eq) target)))
@@ -155,10 +164,10 @@
        (assemble ()
          ;; Is it a fixnum?
          (move temp value)
-         (%test-fixnum temp fixnum nil)
+         (%test-fixnum temp nil fixnum nil)
 
          ;; If not, is it an other pointer?
-         (test-type value nope t (other-pointer-lowtag) :temp temp)
+         (test-type value temp nope t (other-pointer-lowtag))
          ;; Get the header.
          (loadw temp value 0 other-pointer-lowtag)
          ;; Is it one?
@@ -192,7 +201,7 @@
   (:arg-types unsigned-num)
   (:translate fixnump)
   (:conditional :eq)
-  (:generator 5
+  (:generator 3
     (inst tst value (ash (1- (ash 1 (- n-word-bits
                                    n-positive-fixnum-bits)))
                      n-positive-fixnum-bits))))
@@ -203,7 +212,7 @@
   (:info)
   (:arg-types signed-num)
   (:translate fixnump)
-  (:generator 5
+  (:generator 3
     (inst adds temp value value)))
 
 ;;; MOD type checks
@@ -328,7 +337,7 @@
            (is-symbol-label (if not-p drop-thru target)))
       (inst cmp value null-tn)
       (inst b :eq is-symbol-label)
-      (test-type value target not-p (symbol-widetag) :temp temp)
+      (test-type value temp target not-p (symbol-widetag))
       (emit-label drop-thru))))
 
 (define-vop (consp type-predicate)
@@ -338,5 +347,5 @@
            (is-not-cons-label (if not-p target drop-thru)))
       (inst cmp value null-tn)
       (inst b :eq is-not-cons-label)
-      (test-type value target not-p (list-pointer-lowtag) :temp temp)
+      (test-type value temp target not-p (list-pointer-lowtag))
       (emit-label drop-thru))))
