@@ -150,10 +150,9 @@
   (let ((debug-info (debug-info-for-component component)))
     (let* ((2comp (component-info component))
            (constants (ir2-component-constants 2comp))
+           (nboxed (align-up (length constants) sb!c::code-boxed-words-align))
            (code-obj (allocate-code-object
-                      (or #!+immobile-code (eq *compile-to-memory-space* :immobile))
-                      (align-up (length constants) sb!c::code-boxed-words-align)
-                      length)))
+                      (component-mem-space component) nboxed length)))
 
       ;; The following operations need the code pinned:
       ;; 1. copying into code-instructions (a SAP)
