@@ -8,6 +8,15 @@
 ;;;; files for more information.
 
 (in-package "SB!VM")
+
+(defconstant sb!assem:assem-scheduler-p t)
+(defconstant sb!assem:+inst-alignment-bytes+ 4)
+
+(defconstant +backend-fasl-file-implementation+ :mips)
+
+  ;; The o32 ABI specifies 4k-64k as page size. We have to pick the
+  ;; maximum since mprotect() works only with page granularity.
+(defconstant +backend-page-bytes+ 65536)
 
 ;;;; Machine Architecture parameters:
 (eval-when (:compile-toplevel :load-toplevel :execute)
