@@ -80,7 +80,7 @@
   (:generator 5
     (let ((error (generate-error-code vop 'invalid-array-index-error
                                       array bound index)))
-      (%test-fixnum index error t :temp temp)
+      (%test-fixnum index temp error t)
       (inst cmplw index bound)
       (inst bge error))))
 
