@@ -29,6 +29,7 @@
 #include "interr.h" /* for lose() */
 
 extern const char *widetag_names[];
+extern struct weak_pointer *weak_pointer_chain; /* in gc-common.c */
 
 /// Enable extra debug-only checks if DEBUG
 #ifdef DEBUG
@@ -82,41 +83,59 @@ do {                                                                   \
 #define FUN_SELF_FIXNUM_TAGGED 0
 #endif
 
+/* Code component trailer words:
+ *                                                   v code size
+ *      | fun_offset | fun_offset | .... | N-entries |
+ *                                       ^
+ *                 fun_table pointer ---/
+ *
+ * The fun_table pointer is aligned at a 4-byte boundary.
+ */
+static inline unsigned int*
+code_fun_table(struct code* code) {
+  return (unsigned int*)
+      ((char*)code + code_header_words(code->header) * N_WORD_BYTES
+       // Cast out high 32 bits of code_size if lispobj is 64 bits.
+       + fixnum_value((uint32_t)code->code_size) - sizeof (uint16_t));
+}
 static inline unsigned short
-#ifdef LISP_FEATURE_64_BIT
-code_n_funs(struct code* code) { return ((code)->header >> 32) & 0x7FFF; }
-#define FIRST_SIMPLE_FUN_OFFSET(code) ((code)->header >> 48)
-#else
-code_n_funs(struct code* code) { return fixnum_value((code)->n_entries) & 0x3FFF; }
-#define FIRST_SIMPLE_FUN_OFFSET(code) ((code)->n_entries >> 16)
-#endif
+code_n_funs(struct code* code) {
+    // immobile space filler objects appear to be code but have no simple-funs.
+    // Should probably consider changing the widetag to FILLER_WIDETAG.
+    return code_header_words(code->header)
+        ? *((unsigned short*)code_fun_table(code)) >> 4 : 0;
+}
 
-#define is_vector_subtype(header, val) ((HeaderValue(header) & 3) == subtype_##val)
+// Test for presence of a bit in vector's header.
+// As a special case, if 'val' is 0, then test for all bits clear.
+#define is_vector_subtype(header, val) \
+  (subtype_##val ? (HeaderValue(header) & subtype_##val) : \
+   !(HeaderValue(header) & 7))
+
+// Mask out the fullcgc mark bit when asserting header validity
+#define UNSET_WEAK_VECTOR_VISITED(v) \
+  gc_assert((v->header & 0xffff) == \
+    (((subtype_VectorWeakVisited|subtype_VectorWeak) << N_WIDETAG_BITS) \
+     | SIMPLE_VECTOR_WIDETAG)); \
+  v->header ^= subtype_VectorWeakVisited << N_WIDETAG_BITS
 
 // Iterate over the native pointers to each function in 'code_var'
 // offsets are stored as the number of bytes into the instructions
 // portion of the code object at which the simple-fun object resides.
 // We use bytes, not words, because that's what the COMPUTE-FUN vop expects.
-// But the offsets could be compressed further if we chose to use words,
-// which might allow storing them as (unsigned-byte 16),
-// as long as provision is made for ultra huge simple-funs. (~ .5MB)
-//
-// Note that the second assignment to _offset_ is OK: while it technically
-// oversteps the bounds of the indices of the fun offsets, it can not
-// run off the end of the code.
 #define for_each_simple_fun(index_var,fun_var,code_var,assertp,guts)        \
   { int _nfuns_ = code_n_funs(code_var);                                    \
     if (_nfuns_ > 0) {                                                      \
       char *_insts_ = (char*)(code_var) +                                   \
         (code_header_words((code_var)->header)<<WORD_SHIFT);                \
       int index_var = 0;                                                    \
-      int _offset_ = FIRST_SIMPLE_FUN_OFFSET(code_var);                     \
+      unsigned int* _offsets_ = code_fun_table(code_var) - 1;               \
       do {                                                                  \
-       struct simple_fun* fun_var = (struct simple_fun*)(_insts_+_offset_); \
+       struct simple_fun* fun_var                                           \
+           = (struct simple_fun*)(_insts_ + _offsets_[-index_var]);         \
        if (assertp)                                                         \
          gc_assert(widetag_of(fun_var->header)==SIMPLE_FUN_WIDETAG);        \
        guts ;                                                               \
-       _offset_ = ((unsigned int*)_insts_)[index_var];                      \
       } while (++index_var < _nfuns_);                                      \
   }}
 
@@ -127,6 +146,8 @@ code_n_funs(struct code* code) { return fixnum_value((code)->n_entries) & 0x3FFF
  * struct page in gencgc-internal.h. These constants are used in gc-common,
  * so they can't easily be made gencgc-only */
 #define FREE_PAGE_FLAG        0
+#define PAGE_TYPE_MASK        7 // mask out the 'single-object flag'
+/* Note: MAP-ALLOCATED-OBJECTS expects this value to be 1 */
 #define BOXED_PAGE_FLAG       1
 #define UNBOXED_PAGE_FLAG     2
 #define OPEN_REGION_PAGE_FLAG 8
@@ -183,10 +204,12 @@ static inline lispobj funinstance_layout(lispobj* instance_ptr) { // native ptr
     return instance_ptr[3];
 }
 // No layout in simple-fun or closure, because there are no free bits
-static inline lispobj function_layout(lispobj* fun_ptr) { // native ptr
+static inline lispobj
+function_layout(lispobj __attribute__((unused)) *fun_ptr) { // native ptr
     return 0;
 }
-static inline void set_function_layout(lispobj* fun_ptr, lispobj layout) {
+static inline void set_function_layout(lispobj __attribute__((unused)) *fun_ptr,
+                                       lispobj __attribute__((unused)) layout) {
     lose("Can't assign layout");
 }
 #endif
@@ -207,25 +230,36 @@ extern boolean positive_bignum_logbitp(int,struct bignum*);
  * neither of which materially impact garbage collection. */
 
 extern lispobj fdefn_callee_lispobj(struct fdefn *fdefn);
-
-#elif defined(LISP_FEATURE_IMMOBILE_SPACE)
-
-static inline lispobj fdefn_callee_lispobj(struct fdefn *fdefn) {
-    extern unsigned int asm_routines_end;
-    return (lispobj)fdefn->raw_addr -
-      ((uword_t)fdefn->raw_addr < (uword_t)asm_routines_end ? 0 : FUN_RAW_ADDR_OFFSET);
-}
+extern lispobj virtual_fdefn_callee_lispobj(struct fdefn *fdefn,uword_t);
 
 #else
 
-static inline boolean points_to_readonly_space(uword_t ptr) {
+static inline lispobj points_to_asm_routine_p(uword_t ptr) {
+# if defined(LISP_FEATURE_IMMOBILE_SPACE)
+    // Lisp assembly routines are in varyobj space, not readonly space
+    extern unsigned int asm_routines_end;
+    return ptr < (uword_t)asm_routines_end;
+# else
     return READ_ONLY_SPACE_START <= ptr && ptr < READ_ONLY_SPACE_END;
+# endif
 }
 static inline lispobj fdefn_callee_lispobj(struct fdefn *fdefn) {
     return (lispobj)fdefn->raw_addr -
-      (points_to_readonly_space((uword_t)fdefn->raw_addr) ? 0 : FUN_RAW_ADDR_OFFSET);
+      (points_to_asm_routine_p((uword_t)fdefn->raw_addr) ? 0 : FUN_RAW_ADDR_OFFSET);
 }
 
 #endif
 
+#ifdef LISP_FEATURE_IMMOBILE_SPACE
+#include "genesis/layout.h"
+#define LAYOUT_SIZE (sizeof (struct layout)/N_WORD_BYTES)
+/// First 5 layouts: T, FUNCTION, STRUCTURE-OBJECT, LAYOUT, PACKAGE
+/// (These #defines ought to be emitted by genesis)
+#define LAYOUT_OF_FUNCTION ((FIXEDOBJ_SPACE_START+1*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
+#define LAYOUT_OF_LAYOUT   ((FIXEDOBJ_SPACE_START+3*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
+#define LAYOUT_OF_PACKAGE  ((FIXEDOBJ_SPACE_START+4*LAYOUT_ALIGN)|INSTANCE_POINTER_LOWTAG)
+#endif
+
+boolean valid_widetag_p(unsigned char widetag);
+
 #endif /* _GC_INTERNAL_H_ */
