@@ -34,51 +34,55 @@
                                           (1- (length string)))))) ; chop final newline
       (assert (string= line expect)))))
 
+(defun check-does-not-assemble (instruction)
+  (handler-case (test-assemble instruction "")
+    (error nil)
+    (:no-error (x) x (error "Should not assemble"))))
+
 (with-test (:name :assemble-movti-instruction :skipped-on (not :x86-64))
   (flet ((test-movnti (dst src expect)
            (test-assemble `(movnti ,dst ,src) expect)))
-    (test-movnti (make-ea :dword :base rdi-tn :disp 57) eax-tn
-                 "0FC34739         MOVNTI [RDI+57], EAX")
-    (test-movnti (make-ea :qword :base rax-tn) r12-tn
-                 "4C0FC320         MOVNTI [RAX], R12")))
+    (test-movnti (ea 57 rdi-tn) eax-tn "0FC34739         MOVNTI [RDI+57], EAX")
+    (test-movnti (ea rax-tn) r12-tn "4C0FC320         MOVNTI [RAX], R12")))
 
 (with-test (:name :assemble-crc32 :skipped-on (not :x86-64))
   ;; Destination size = :DWORD
-  (test-assemble `(crc32 ,eax-tn ,(make-ea :byte  :base rbp-tn))
+  (test-assemble `(crc32 :byte ,eax-tn ,(ea rbp-tn))
                  "F20F38F04500     CRC32 EAX, BYTE PTR [RBP]")
-  (test-assemble `(crc32 ,eax-tn ,(make-ea :word  :base rbp-tn))
+  (test-assemble `(crc32 :byte ,eax-tn (,rcx-tn . :high-byte))
+                 "F20F38F0C5       CRC32 EAX, CH")
+  (test-assemble `(crc32 :byte ,eax-tn ,dil-tn)
+                 "F2400F38F0C7     CRC32 EAX, DIL")
+  (test-assemble `(crc32 :word ,eax-tn ,(ea rbp-tn))
                  "66F20F38F14500   CRC32 EAX, WORD PTR [RBP]")
-  (test-assemble `(crc32 ,eax-tn ,(make-ea :dword :base rbp-tn))
+  (test-assemble `(crc32 :dword ,eax-tn ,(ea rbp-tn))
                  "F20F38F14500     CRC32 EAX, DWORD PTR [RBP]")
   ;; these check that the presence of REX does not per se change the width.
-  (test-assemble `(crc32 ,r9d-tn ,(make-ea :byte  :base r14-tn :index r15-tn))
+  (test-assemble `(crc32 :byte ,r9d-tn ,(ea r14-tn r15-tn))
                  "F2470F38F00C3E   CRC32 R9D, BYTE PTR [R14+R15]")
-  (test-assemble `(crc32 ,r9d-tn ,(make-ea :word  :base r14-tn :index r15-tn))
+  (test-assemble `(crc32 :word ,r9d-tn ,(ea r14-tn r15-tn))
                  "66F2470F38F10C3E CRC32 R9D, WORD PTR [R14+R15]")
-  (test-assemble `(crc32 ,r9d-tn ,(make-ea :dword :base r14-tn :index r15-tn))
+  (test-assemble `(crc32 :dword ,r9d-tn ,(ea r14-tn r15-tn))
                  "F2470F38F10C3E   CRC32 R9D, DWORD PTR [R14+R15]")
   ;; Destination size = :QWORD
-  (test-assemble `(crc32 ,rax-tn ,(make-ea :byte  :base rbp-tn))
+  (test-assemble `(crc32 :byte ,rax-tn ,(ea rbp-tn))
                  "F2480F38F04500   CRC32 RAX, BYTE PTR [RBP]")
-  (test-assemble `(crc32 ,rax-tn ,(make-ea :qword :base rbp-tn))
+  (test-assemble `(crc32 :qword ,rax-tn ,(ea rbp-tn))
                  "F2480F38F14500   CRC32 RAX, QWORD PTR [RBP]")
   ;; now with high regs
-  (test-assemble `(crc32 ,r9-tn ,(make-ea :byte  :base r14-tn :index r15-tn))
+  (test-assemble `(crc32 :byte ,r9-tn ,(ea r14-tn r15-tn))
                  "F24F0F38F00C3E   CRC32 R9, BYTE PTR [R14+R15]")
-  (test-assemble `(crc32 ,r9-tn ,(make-ea :qword :base r14-tn :index r15-tn))
+  (test-assemble `(crc32 :qword ,r9-tn ,(ea r14-tn r15-tn))
                  "F24F0F38F10C3E   CRC32 R9, QWORD PTR [R14+R15]"))
 
-(with-test (:name :assemble-unsigned-qword-to-mem :skipped-on (not :x86-64))
+(with-test (:name :assemble-unsigned-qword-imm-to-mem :skipped-on (not :x86-64))
   ;; unsigned bits cast as signed bits
   (let ((const #xffffffff801234BB))
-    (test-assemble `(mov ,(make-ea :qword :base rcx-tn) ,const)
+    (test-assemble `(mov :qword ,(ea rcx-tn) ,const)
                    "48C701BB341280   MOV QWORD PTR [RCX], -2146290501")
     ;; Do not truncate to just the lower bits
     (dolist (size '(:byte :word :dword))
-      (handler-case (test-assemble `(mov ,(make-ea size :base rcx-tn) ,const)
-                                   "")
-        (type-error nil)
-        (:no-error (x) x (error "Should not assemble"))))))
+      (check-does-not-assemble `(mov ,size ,(ea rcx-tn) ,const)))))
 
 (with-test (:name :unsigned-as-signed-imm8 :skipped-on (not :x86-64))
   ;; PUSH
@@ -92,6 +96,27 @@
   ;; imm16 operand; the encoding length is the same either way.
   (test-assemble `(or ,ax-tn #xfff7) "6683C8F7         OR AX, -9"))
 
+(with-test (:name :assemble-movsx :skipped-on (not :x86-64))
+  ;; source = :BYTE, signed
+  (check-does-not-assemble `(movsx (:byte :byte) ,r8b-tn ,cl-tn))
+  (test-assemble `(movsx (:byte :word)  ,r8w-tn ,cl-tn) "66440FBEC1       MOVSX R8W, CL")
+  (test-assemble `(movsx (:byte :dword) ,r8d-tn ,cl-tn) "440FBEC1         MOVSX R8D, CL")
+  (test-assemble `(movsx (:byte :qword) ,r8-tn  ,cl-tn) "4C0FBEC1         MOVSX R8, CL")
+  ;; source = :BYTE, unsigned
+  (check-does-not-assemble `(movzx (:byte :byte) ,r8b-tn ,cl-tn))
+  (test-assemble `(movzx (:byte :word)  ,r8w-tn ,cl-tn) "66440FB6C1       MOVZX R8W, CL")
+  (test-assemble `(movzx (:byte :dword) ,r8d-tn ,cl-tn) "440FB6C1         MOVZX R8D, CL")
+  (test-assemble `(movzx (:byte :qword) ,r8-tn  ,cl-tn) "440FB6C1         MOVZX R8D, CL") ; R8D, not R8
+  ;; source = :WORD, signed
+  (test-assemble `(movsx (:word :dword) ,r8d-tn ,cx-tn) "440FBFC1         MOVSX R8D, CX")
+  (test-assemble `(movsx (:word :qword) ,r8-tn ,cx-tn)  "4C0FBFC1         MOVSX R8, CX")
+  ;; source = :WORD, unsigned
+  (test-assemble `(movzx (:word :dword) ,r8d-tn ,cx-tn) "440FB7C1         MOVZX R8D, CX")
+  (test-assemble `(movzx (:word :qword) ,r8-tn ,cx-tn)  "440FB7C1         MOVZX R8D, CX") ; R8D, not R8
+  ;; source = :DWORD, signed and unsigned
+  (test-assemble `(movsx (:dword :qword) ,r8-tn ,ecx-tn) "4C63C1           MOVSX R8, ECX")
+  (test-assemble `(movzx (:dword :qword) ,r8-tn ,ecx-tn) "448BC1           MOV R8D, ECX"))
+
 (with-test (:name :disassemble-movabs-instruction :skipped-on (not :x86-64))
   (let* ((bytes (coerce '(#x48 #xA1 8 7 6 5 4 3 2 1
                           #xA1 8 7 6 5 4 3 2 1
@@ -125,7 +150,7 @@
     (try `(bt ,(memref :word)  ,ax-tn)  "660FA34500       BT WORD PTR [$fp], AX")
     (try `(bt ,(memref :dword) ,eax-tn) "0FA34500         BT DWORD PTR [$fp], EAX")
     #+x86-64
-    (try `(bt ,(memref :qword) ,eax-tn) "480FA34500       BT QWORD PTR [$fp], RAX")
+    (try `(bt ,(memref :qword) ,rax-tn) "480FA34500       BT QWORD PTR [$fp], RAX")
     (try `(bt ,(memref :word)  3) "660FBA650003     BT WORD PTR [$fp], 3")
     (try `(bt ,(memref :dword) 3) "0FBA650003       BT DWORD PTR [$fp], 3")
     #+x86-64
@@ -166,6 +191,12 @@
     (try `(add ,rcx-tn ,(memref :qword)) "48034D00         ADD RCX, [$fp]")
     ))
 
+(with-test (:name :disassemble-imul :skipped-on (not (or :x86 :x86-64)))
+  (test-assemble `(imul ,dl-tn)  "F6EA             IMUL DL")
+  (test-assemble `(imul ,cx-tn)  "66F7E9           IMUL CX")
+  (test-assemble `(imul ,ebx-tn) "F7EB             IMUL EBX")
+  (test-assemble `(imul ,edi-tn 92) "6BFF5C           IMUL EDI, EDI, 92"))
+
 (with-test (:name :disassemble-fs-prefix :skipped-on (not (or :x86-64)))
   (let ((bytes (coerce '(#x64 #xF0 #x44 #x08 #x04 #x25 #x00 #x04 #x10 #x20)
                        '(array (unsigned-byte 8) 1)))
@@ -193,3 +224,31 @@
                  (search " SUB-GC" line))
         (setq found t)))
     (assert found)))
+
+(with-test (:name :cast-reg-to-size :skipped-on (not :x86-64))
+  (test-assemble `(mov :byte ,rsi-tn ,rdi-tn)
+                 "408AF7           MOV SIL, DIL")
+  (test-assemble `(movsx (:byte :word) ,rax-tn ,rdi-tn)
+                 "66400FBEC7       MOVSX AX, DIL")
+  (test-assemble `(cmpxchg :byte ,(ea rax-tn) ,rdi-tn)
+                 "400FB038         CMPXCHG [RAX], DIL")
+  (test-assemble `(cmp :byte ,rdi-tn ,rsi-tn)
+                 "4038F7           CMP DIL, SIL")
+  (test-assemble `(not :byte ,rsi-tn)
+                 "40F6D6           NOT SIL")
+  (test-assemble `(rol :byte ,rsi-tn 2)
+                 "40C0C602         ROL SIL, 2")
+  (test-assemble `(test :byte ,rsi-tn 15)
+                 "40F6C60F         TEST SIL, 15")
+  (test-assemble `(movd ,float0-tn ,rax-tn)
+                 "660F6EC0         MOVD XMM0, EAX")
+  (test-assemble `(movq ,float0-tn ,eax-tn)
+                 "66480F6EC0       MOVQ XMM0, RAX"))
+
+(with-test (:name :assemble-high-byte-regs :skipped-on (not :x86-64))
+  (test-assemble `(cmp (,rdx-tn . :high-byte) 1)
+                 "80FE01           CMP DH, 1")
+  (test-assemble `(mov (,edx-tn . :high-byte) (,rcx-tn . :high-byte))
+                 "8AF5             MOV DH, CH")
+  ;; can not use legacy high byte reg in a REX-prefixed instruction
+  (check-does-not-assemble `(movsx (:byte :qword) ,rax-tn (,rbx-tn . :high-byte))))
