@@ -1524,3 +1524,51 @@
          (declare (type (string 1) s))
          (the (or simple-array (member 1/2 "ba" 0 #\3)) s))
     ((#1="a") #1#)))
+
+(with-test (:name :primitive-type-function)
+  (checked-compile-and-assert
+      ()
+      `(lambda (x)
+         (funcall (the (and atom (not null)) x))
+         )
+    ((#'list) nil)
+    (('list) nil)))
+
+(with-test (:name :copyprop-sc-mismatch-between-moves
+            :skipped-on :interpreter)
+  (let ((f (checked-compile
+            '(lambda (f x)
+              (let ((x (the double-float x)))
+                (values (funcall f x) (> x 1d0)))))))
+    (ctu:assert-no-consing (funcall f #'identity 1d0))))
+
+(with-test (:name :infer-iteration-var-type)
+  (let ((f (checked-compile
+            '(lambda (s)
+              (declare ((integer 1 2) s))
+              (let ((r 16))
+                (loop for i from 16 below 32 by s
+                      do (setf r i))
+                r)))))
+    (assert (equal (sb-impl::%simple-fun-type f)
+                   '(function ((integer 1 2)) (values (integer 16 31) &optional))))))
+
+(with-test (:name :delay-transform-until-constraint-loop)
+  (checked-compile-and-assert
+      ()
+      `(lambda (str)
+         (declare (string str))
+         (when (plusp (length str))
+           (make-array (1- (length str))
+                       :element-type (array-element-type str)
+                       :displaced-to str)))
+    (("abc") "ab" :test #'equal)))
+
+(with-test (:name :lambda-var-ref-lvar-loop)
+  (checked-compile-and-assert
+      ()
+      `(lambda ()
+         (labels ((z (a)
+                    (when (>= 0 (the integer a))
+                      (values #'z a))))))
+    (() nil)))
