@@ -26,22 +26,27 @@ run_sbcl <<EOF >$tmpfilename
 (setq sb-ext:*evaluator-mode* :compile)
 (defvar *fred*)
 (defstruct foo a)
+
+(defun scrubstack () 
+  (sb-int:dx-let ((b (make-array 20))) (eval b))
+  (sb-sys:scrub-control-stack))
+
 (defun test1 (wp obj root)
   (let ((*fred* (list (make-foo :a (list (vector #xfeefa (list obj)))))))
     (setq obj nil) ; so OBJ is not found as a stack reference
     (ecase root
       (:tls
-       (sb-sys:scrub-control-stack)
+       (scrubstack)
        (sb-ext::gc-and-search-roots wp))
       (:bindings ; bind *FRED* again so the old value is on the binding stack
        (let ((*fred* 1))
-         (sb-sys:scrub-control-stack)
+         (scrubstack)
          (sb-ext::gc-and-search-roots wp)))
       (:stack
        ; put the OBJ back on the control stack
        ; and also ensure that *FRED* is not a root.
        (setq obj *fred* *fred* nil)
-       (sb-sys:scrub-control-stack)
+       (scrubstack)
        (sb-ext::gc-and-search-roots wp)))))
 
 (let ((wp (make-weak-pointer (list 1 2 3 4))))
